# babelfish.tcl - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sf.net>
#
# Translate the selection using Babelfish. The translated data is copied from
# the babelfish web page and displayed in a Tk dialog.
#
# $Id: babelfish.tcl,v 1.1 2005/10/20 21:35:57 pat Exp $

namespace eval babelfish {

    custom::defvar options(use_babelfish) 1 \
        [::msgcat::mc "Enable BabelFish translation menus."] \
        -group Chat -type boolean

    custom::defvar options(babelfish_replace) 0 \
        [::msgcat::mc "Replace selection when translated."] \
        -group Chat -type boolean

    custom::defvar options(babelfish_url) \
        "http://babelfish.altavista.com/babelfish/tr" \
        -group Chat -type string

    hook::add connected_hook [namespace current]::init 90
    #hook::add chat_win_popup_menu_hook \
        [namespace current]::add_chat_win_popup_menu 10

    variable Auth
    if {![info exists Auth]} {
        set Auth {}
    }
}

proc babelfish::init {connid} {
    global loginconf
    variable options
    variable Auth
    if {$options(use_babelfish) 
        && [string length $options(babelfish_url)] > 0} {

        package require http
        if {![info exists loginconf(useragent)]} {
            set loginconf(useragent) [http::config -useragent]
        }
        http::config \
            -proxyfilter [namespace current]::proxyfilter \
            -useragent $loginconf(useragent)
        
        if {[package provide tls] != {}} {
            http::register https 443 ::tls::socket
        }
        
        if {$loginconf(useproxy)} {
            if {[string length $loginconf(httplogin)] > 0} {
                set Auth [list Proxy-Authorization \
                       [concat Basic [base64::encode \
                           "$loginconf(httplogin):$loginconf(httppassword)"]]]
            }
        }
        
        set tok [::http::geturl $options(babelfish_url) -headers $Auth \
                     -command [list [namespace current]::fetchurldone \
                                   [namespace current]::initdone]]
    }
}

proc babelfish::initdone {tok} {
    variable languages
    set languages {}
    debugmsg babelfish "Babelfish init done."
    set data [http::data $tok]
    if {[regexp {<select name="lp"[^>]*?>(.*?)</select>} $data -> r]} {
        set lst [split [string trim $r] \n]
        foreach option $lst {
            regexp {<option value="(.*?)"[^>]*>(.*?)</option>} \
                    $option -> value label
            lappend languages $label [split $value _]
        }

        add_to_menu

    } else {
        debugmsg babelfish "babelfish init received no data"
    }
}

proc babelfish::proxyfilter {host} {
    global loginconf
    if {[string match "localhost*" $host] || [string match "127.*" $host]} {
        return {}
    }
    if {$loginconf(useproxy)} {
        return [list $loginconf(httpproxy) $loginconf(httpproxyport)]
    }
    return {}
}

proc babelfish::fetchurldone {cmd tok} {
    switch -- [::http::status $tok] {
	ok {
            if {[::http::ncode $tok] >= 500} {
                debugmsg tclchat "error [::http::code $tok]"
            } elseif {[http::ncode $tok] >= 400} {
                debugmsg tclchat "error [::http::code $tok]"
            } else {
                $cmd $tok
            }
	}
        reset   { debugmsg tclchat "reset" }
        timeout { debugmsg tclchat "timeout" }
        error   { debugmsg tclchat "error [::http::error $tok]" }
        default { debugmsg tclchat "????? [::http::error $tok]" }
    }
    ::http::cleanup $tok
}

proc babelfish::translateSel {from to} {
    if {![catch {selection get} msg]} {
        debugmsg babelfish "translate: $from $to \"$msg\""
        translate $from $to $msg
    }
}

proc babelfish::translate {from to text} {
    variable options
    variable Auth
    append op $from _ $to
    set text [string map {\n \u00b6} $text]
    set query [http::formatQuery tt urltext urltext $text lp $op]
    set hdrs $Auth
    lappend hdrs "Accept-Charset" "ISO-8859-1,utf-8"
    set tok [http::geturl $options(babelfish_url) \
             -query $query -headers $hdrs \
                 -command [list [namespace current]::fetchurldone \
                               [namespace current]::translateDone]]
}

proc babelfish::paste_short_url {} {
    variable Auth
    set url {}
    if {[info command clipboard] ne {}} {
        catch {set url [clipboard get]}
    } else {
        catch {set url [selection get]}
    }
    if {[string length $url] < 1} {
        tk_messageBox -message "Select a url to shorten. FIX ME: request input"
        return
    }

    set hdrs $Auth; lappend hdrs "Accept-Charset" "ISO-8859-1,utf-8"
    set tok [http::geturl "http://metamark.net/api/rest/simple?long_url=$url" \
                 -headers $hdrs -command [list [namespace current]::fetchurldone \
                                              [namespace current]::paste_short_url_done]]
}

proc babelfish::paste_short_url_done {tok} {
    set short [lindex [split [http::data $tok] "\n"] 0]
}
    
proc babelfish::translateDone {tok} {
    variable options
    set text {}
    set r [regexp {<td.*?class=s><div.*?>(.*)</div>} \
               [::http::data $tok] -> text]
    set text [string map {\u00b6 \n} [string trim $text]]
    debugmsg babelfish "translation \"$text\""
    if {!$r} {
        set text "Translation returned no matching data."
    }
    
    if {![winfo exists .translation]} {
        set t [toplevel .translation]
        text $t.t -width 60 -height 6 -font $::font \
            -yscrollcommand [list $t.s set]
        scrollbar $t.s -command [list $t.t yview]
        button $t.b0 -text Dismiss -font $::font \
            -command [list wm withdraw $t]
        button $t.b1 -text Clear   -font $::font \
            -command [list $t.t delete 0.0 end]
        grid $t.t    -    $t.s  -sticky news
        grid $t.b1 $t.b0  -     -sticky e
        grid rowconfigure $t 0 -weight 1
        grid columnconfigure $t 0 -weight 1
        tk::PlaceWindow $t parent .
    }
    .translation.t insert end "$text\n" {}
    .translation.t see end
    wm deiconify .translation

    if ($options(babelfish_replace)) {
        .translation.t delete 0.0 end
    }
}

proc babelfish::add_to_menu {} {
    variable languages
    
    if {[catch {set menu [.mainframe getmenu plugins]}]} {
        return
    }

    if {[catch {$menu index [::msgcat::mc "Translate"]} ndx]} {
        $menu add cascade -label [::msgcat::mc "Translate"] \
            -menu [menu ${menu}.tr]
    } else {
        ${menu}.tr delete 0 end
    }
    foreach {label code} $languages {
        ${menu}.tr add command -label $label \
            -command [concat [namespace current]::translateSel $code]
    }
}

proc babelfish::add_chat_win_popup_menu {m chatwin X Y x y} {
    variable options

    if {! $options(use_babelfish)} {return}
    if {[lempty [$W tag ranges sel]]} {
        set state disabled
    } else {
        set state normal
    }

#    $m add command -label [::msgcat::mc "Edit nick color..."] \
        -command [list [namespace current]::edit_nick_color $chatwin $nick]
    
#    .mbar.help.tr add command -label $label \
        -command [concat [namespace current]::translateSel $value]
}
